@@ -1,7 +1,6 @@
 
 
 #include "DDC.h"
-#include "cellMCD.h"
 
 /*************************************/
 /*       Main DDCcore function       */
@@ -758,326 +757,5 @@ Rcpp::List findCellPath_cpp(arma::mat & predictors,
 }
 
 
-// cellMCD //
-
-// [[Rcpp::export]]
-Rcpp::List allpreds_cpp(arma::mat& X,
-                        arma::mat& S,
-                        arma::vec& mu,
-                        arma::umat& W) {
-  try
-  {
-    
-    arma::uword n = W.n_rows;
-    arma::uword d = W.n_cols;
-    //arma::mat Sinv = arma::inv_sympd(arma::symmatu(S));
-    arma::mat Sinv = arma::inv(S);
-    arma::mat preds(n, d);
-    arma::mat cvars(n, d);
-    
-    
-    std::unordered_map<std::string, arma::uvec> Wmap;
-    uniqueRows(W, Wmap);
-    
-    
-    for (std::pair<std::string, arma::uvec> s : Wmap) {
-      
-      
-      arma::uvec myrows = s.second; // all indices with rowpattern(string) s
-      arma::urowvec w = W.row(myrows(0)); // the actual rowpattern
-      
-      arma::uvec obs = arma::find(w);
-      arma::uvec mis = arma::find(1 - w);
-      
-      if (mis.n_elem < d) {
-        if (mis.n_elem > 0) {
-          
-          arma::mat Sjinv = subinverse_cpp(S, Sinv, obs);
-          
-          arma::vec cvar = arma::diagvec(S(mis, mis) -
-            S(mis, obs) * Sjinv * S(obs, mis));
-          for (arma::uword i = 0; i < myrows.n_elem; i++) {
-            arma::uvec rowi(1);
-            rowi(0) = myrows(i);
-            cvars(rowi, mis) = cvar.t();
-            preds(rowi, mis) = mu(mis).t() + (X(rowi, obs) - mu(obs).t()) * Sjinv * S(obs, mis);
-          }
-          
-          // now compute preds and cvars for all available entries
-          //
-          
-          if (obs.n_elem == 1) {
-            for (arma::uword i = 0; i < myrows.n_elem; i++) {
-              arma::uvec rowi(1);
-              rowi(0) = myrows(i);
-              preds(rowi, obs) = mu(obs);
-              cvars(rowi, obs) = S(obs, obs);
-            }
-          } else {
-            for (arma::uword j = 0; j < obs.n_elem; j++) {
-              arma::uvec obsj(1);
-              obsj(0) = obs(j);
-              arma::uvec helps = obs(arma::find(obs != obs(j)));
-              arma::mat M2inv = subinverse_cpp(S, Sinv, helps);
-              
-              arma::mat cvar = S(obsj, obsj) - S(obsj, helps) *
-                M2inv * S(helps, obsj);
-              
-              for (arma::uword i = 0; i < myrows.n_elem; i++) {
-                arma::uvec rowi(1);
-                rowi(0) = myrows(i);
-                cvars(rowi, obsj) = cvar;
-                preds(rowi, obsj) = mu(obsj).t() +
-                  (X(rowi, helps) - mu(helps).t()) * M2inv * S(helps, obsj);
-              }
-            }
-          }
-          
-        } else { //no missings in this pattern
-          
-          arma:: vec cvar = (1 / arma::diagvec(Sinv));
-          
-          for (arma::uword i = 0; i < myrows.n_elem; i++) {
-            cvars.row(myrows(i)) =  cvar.t();
-          }
-          
-          for (arma::uword j = 0; j < d; j++) {
-            arma::uvec jvec(1);
-            jvec(0) = j;
-            arma::uvec j_neg = arma::regspace<arma::uvec>(0, d - 1);
-            j_neg.shed_rows(jvec);
-            
-            arma::mat Sjinv = subinverse_cpp(S, Sinv, j_neg);
-            arma::mat Xtemp = X(myrows, j_neg);
-            Xtemp.each_row() -= mu(j_neg).t();
-            
-            preds(myrows, jvec) = (arma::as_scalar(mu(j)) +
-              Xtemp * Sjinv * S(j_neg, jvec));
-          }
-        }
-      } else { // all elements of this pattern are missing
-        for (arma::uword i = 0; i < myrows.n_elem; i++) {
-          preds.row(myrows(i)) = mu.t();
-          cvars.row(myrows(i)) =  arma::diagvec(Sinv).t();
-        }
-      }
-      
-    }
-    
-    
-    return Rcpp::List::create(Rcpp::Named("preds") = preds,
-                              Rcpp::Named("cvars") = cvars);
-  } catch( std::exception& __ex__ )
-  {
-    forward_exception_to_r( __ex__ );
-  } catch(...)
-  {
-    ::Rf_error( "c++ exception " "(unknown reason)" );
-  }
-  
-  return Rcpp::wrap(NA_REAL);
-}
-
-
-
-// [[Rcpp::export]]
-double Objective_cpp(arma::mat &X,
-                     arma::umat &W,
-                     arma::vec &mu,
-                     arma::mat &Sigma,
-                     arma::mat &Sigmai) {
-  try
-  {
-    
-    double objective = 0;
-    double objective_add = 0;
-    
-    
-    std::unordered_map<std::string, arma::uvec> Wmap;
-    uniqueRows(W, Wmap);
-    
-    
-    for (std::pair<std::string, arma::uvec> s : Wmap) {
-      
-      arma::uvec myrows = s.second; // all indices with rowpattern(string) s
-      arma::urowvec w = W.row(myrows(0)); // the actual rowpattern
-      
-      arma::uvec obs = arma::find(w);
-      arma::uvec mis = arma::find(1 - w);
-      
-      arma::vec subMu = mu(obs);
-      arma::mat subSigma = Sigma(obs, obs);
-      arma::mat invsubSigma = subinverse_cpp(Sigma, Sigmai, obs);
-      
-      
-      arma::mat Xtemp = X(myrows, obs);
-      Xtemp.each_row() -= subMu.t();
-      arma::mat MD = arma::sum((Xtemp * invsubSigma) % Xtemp, 1);
-      objective_add = arma::as_scalar(arma::sum(MD + 
-        arma::log_det_sympd(arma::symmatu(subSigma)) + 
-        std::log(2 * arma::datum::pi) * (obs.n_elem)));
-      objective    = objective + objective_add;
-    } 
-    
-    return(objective);
-  } catch( std::exception& __ex__ )
-  {
-    forward_exception_to_r( __ex__ );
-  } catch(...)
-  {
-    ::Rf_error( "c++ exception " "(unknown reason)" );
-  }
-  
-  return NA_REAL;
-}
-
-
-// [[Rcpp::export]]
-arma::umat updateW_cpp(const arma::mat &X,
-                       arma::umat W,
-                       const arma::vec &mu,
-                       const arma::mat &Sigma,
-                       const arma::mat &Sigmai,
-                       const arma::vec &lambda,
-                       const arma::uword &h) {
-  try
-  {
-    
-    arma::uword n = W.n_rows;
-    arma::uword d = W.n_cols;
-    arma::uvec ord = arma::stable_sort_index(arma::sum(W, 0));
-    arma::vec Delta(n);
-    arma::uvec goodCells;
-    arma::uvec wnew(n);
-    arma::uvec Delta_rank(n);
-    for (arma::uword j = 0; j < d; j++) {
-      arma::uword jtemp = ord(j);
-      Delta = Deltacalc_cpp(X, W, Sigma, Sigmai, mu, jtemp + 1);
-      goodCells = arma::find(Delta <= lambda(jtemp));
-      if (goodCells.n_elem < h) {
-        Delta_rank = arma::stable_sort_index(Delta - lambda(jtemp));
-        goodCells = Delta_rank.head(h);
-      }
-      wnew.zeros();
-      wnew(goodCells).fill(1);
-      W.col(jtemp) = wnew;
-    }
-    return(W);
-  } catch( std::exception& __ex__ )
-  {
-    forward_exception_to_r( __ex__ );
-  } catch(...)
-  {
-    ::Rf_error( "c++ exception " "(unknown reason)" );
-  }
-  arma::umat error_out(1,1);
-  error_out.fill(0);
-  return error_out;
-}
-
-
-
-
-// [[Rcpp::export]]
-Rcpp::List Cstep(const arma::mat &X,
-                 arma::umat W,
-                 const arma::vec &mu,
-                 const arma::mat &Sigma,
-                 const arma::mat &Sigmai,
-                 const arma::vec &lambda,
-                 const arma::uword &h,
-                 const bool fixedCenter) {
-  try
-  {
-    // First update W
-    
-    arma::uword n = W.n_rows;
-    arma::uword d = W.n_cols;
-    arma::uvec ord = arma::stable_sort_index(arma::sum(W, 0));
-    arma::vec Delta(n);
-    arma::uvec goodCells;
-    arma::uvec wnew(n);
-    arma::uvec Delta_rank(n);
-    for (arma::uword j = 0; j < d; j++) {
-      arma::uword jtemp = ord(j);
-      Delta = Deltacalc_cpp2(X, W, Sigma, Sigmai, mu, jtemp + 1);
-      goodCells = arma::find(Delta <= lambda(jtemp));
-      if (goodCells.n_elem < h) {
-        Delta_rank = arma::stable_sort_index(Delta - lambda(jtemp));
-        goodCells = Delta_rank.head(h);
-      }
-      wnew.zeros();
-      wnew(goodCells).fill(1);
-      W.col(jtemp) = wnew;
-    }
-    
-    
-    
-    // Now execute one EM step:
-    // first compute Ximp
-    
-    arma::mat Ximp = X;
-    
-    std::unordered_map<std::string, arma::uvec> Wmap;
-    uniqueRows(W, Wmap);
-    
-    arma::mat bias(d, d, arma::fill::zeros);
-    
-    for (std::pair<std::string, arma::uvec> s : Wmap) {
-      
-      
-      arma::uvec myrows = s.second; // all indices with rowpattern(string) s
-      arma::urowvec w = W.row(myrows(0)); // the actual rowpattern
-      
-      arma::uvec obs = arma::find(w);
-      arma::uvec mis = arma::find(1 - w);
-      
-      if (mis.n_elem < d) {
-        if (mis.n_elem > 0) {
-          
-          arma::mat Sigmai_temp = subinverse_cpp(Sigmai, Sigma, mis); //solve(Sigmai[mis, mis])  
-          bias(mis,mis) += (Sigmai_temp * (double)myrows.n_elem);
-   
-          for (arma::uword i = 0; i < myrows.n_elem; i++) {
-            arma::uvec rowi(1);
-            rowi(0) = myrows(i);
-            Ximp(rowi, mis) = mu(mis).t() - (Sigmai_temp * Sigmai(mis, obs) *(X(rowi, obs).t() - mu(obs))).t();
-          }
-        }
-      } else { // all elements of this pattern are missing
-        bias += (Sigma * (double)myrows.n_elem);
-        for (arma::uword i = 0; i < myrows.n_elem; i++) {
-          Ximp.row(myrows(i)) = mu.t();
-        }
-      }
-    }
-    
-    arma::vec newmu = mu;
-    if (!fixedCenter) {
-      newmu = mean(Ximp, 0).t();
-    }
-    
-    bias /= (double)n;
-    
-    arma::mat newSigma = bias;
-    if (!fixedCenter) {
-      newSigma += arma::cov(Ximp, 1);
-    } else {
-      newSigma += (Ximp.t() * Ximp) / (double)n ;
-    }
-    
-    return Rcpp::List::create(Rcpp::Named("W") = W,
-                              Rcpp::Named("mu") = newmu,
-                              Rcpp::Named("Sigma") = newSigma);
-  } catch( std::exception& __ex__ )
-  {
-    forward_exception_to_r( __ex__ );
-  } catch(...)
-  {
-    ::Rf_error( "c++ exception " "(unknown reason)" );
-  }
-  return Rcpp::List::create(Rcpp::Named("error") = true);
-}
-
 
 
